@@ -7,13 +7,10 @@ from contrast.utils.context_tracker import ContextTracker
 
 from contrast.assess_extensions import cs_str
 
-
 __version__ = get_distribution("contrast-agent").version
 
-
 CS__CONTEXT_TRACKER = ContextTracker()
 
-
 STRING_TRACKER = StringTracker()
 
 # PERF: These values are constant for the lifetime of the agent,
@@ -25,3 +22,10 @@ SORTED_SYS_PATH = sorted(sys.path, key=len, reverse=True)
 # --- import aliases ---
 
 from contrast.agent.assess.utils import get_properties  # noqa
+
+TELEMETRY = None
+
+
+def telemetry_disabled():
+    OPT_OUT = os.environ.get("CONTRAST_AGENT_TELEMETRY_OPTOUT", False)
+    return OPT_OUT and OPT_OUT.lower() in ("1", "true")
