@@ -101,7 +101,7 @@ def build_finding(
     if target_properties is not None:
         events = [event.to_dtm() for event in target_properties.events]
 
-        target_props = target_properties.properties
+        target_props = target_properties.dynamic_source_metadata
         if target_props:
             for key, value in target_props.items():
                 properties[key] = value
