@@ -64,6 +64,29 @@ def build_assess_method(original_method, patch_policy):
     return add_watermark(assess_method_wrapper(original_method))
 
 
+def build_assess_async_method(original_method, patch_policy):
+    """
+    Build a generic async method which instruments original_method.
+
+    :param original_method: method to call for result
+    :param patch_policy: PatchLocationPolicy containing all policy nodes for this patch
+    :return: Newly created async patch function
+    """
+
+    async def assess_method(*args, **kwargs):
+        context = contrast.CS__CONTEXT_TRACKER.current()
+        result = None
+
+        try:
+            result = await original_method(*args, **kwargs)
+        finally:
+            analyze(context, patch_policy, result, args, kwargs)
+
+        return result
+
+    return add_watermark(assess_method)
+
+
 def build_assess_classmethod(original_method, patch_policy):
     """
     Creates method to replace old method and call our assess code with the original method
@@ -110,6 +133,18 @@ def build_assess_deadzone(original_method, patch_policy):
     return add_watermark(assess_deadzone)
 
 
+def build_assess_async_deadzone(original_method, patch_policy):
+    """
+    The equivalent of `build_assess_deadzone`, but for async
+    """
+
+    async def assess_deadzone(*args, **kwargs):
+        with scope.contrast_scope():
+            return await original_method(*args, **kwargs)
+
+    return add_watermark(assess_deadzone)
+
+
 def build_assess_property_fget(original_property_name, patch_policy):
     """
     Creates property getter to replace old property and call assess code for analysis
@@ -147,8 +182,10 @@ def apply_cached_property(cls_or_module, patch_policy, property_name, orig_prope
 
     try:
         setattr(cls_or_module, property_name, proxied_property)
-    except Exception:
-        logger.exception("Failed to apply patch to cached_property: %s", property_name)
+    except Exception as ex:
+        logger.debug(
+            "Failed to apply patch to cached_property: %s", property_name, exc_info=ex
+        )
 
     return True
 
@@ -170,7 +207,9 @@ class CachedPropertyProxy(wrapt.ObjectProxy):
             # Self is the only arg that seems to be relevant for policy/reporting
             args = (__cs_self.__wrapped__,)
             analyze(context, __cs_self.cs__patch_policy, result, args, {})
-        except Exception:
-            logger.exception("Failed to apply policy for %s", __cs_self.cs__attr_name)
+        except Exception as ex:
+            logger.debug(
+                "Failed to apply policy for %s", __cs_self.cs__attr_name, exc_info=ex
+            )
 
         return result
