@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2021 Contrast Security, Inc.
+# Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from contrast.utils.assess.duck_utils import len_or_zero
 
@@ -16,4 +16,4 @@ class Preshift(object):
         self.kwargs = call_kwargs or {}
 
     def __repr__(self):
-        return "<Preshift({}, {}, {})>".format(self.obj, self.args, self.kwargs)
+        return f"<Preshift({self.obj}, {self.args}, {self.kwargs})>"
