@@ -3,7 +3,7 @@
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from contrast.agent.assess.utils import get_properties
 from contrast.agent.assess.policy.propagators.base_propagator import BasePropagator
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 
 from contrast.extern import structlog as logging
 
@@ -20,7 +20,7 @@ class DBWritePropagator(BasePropagator):
     """
 
     @property
-    @fail_safely(
+    @fail_quietly(
         "Failed to determine if DB_WRITE needs propagation", return_value=False
     )
     def needs_propagation(self):
