@@ -3,7 +3,7 @@
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from .secure_flag_rule import SecureFlagRuleMixin
 from contrast.agent.assess.rules.config.base_config_rule import BaseConfigRule
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 
 
 class FalconSecureFlagRule(SecureFlagRuleMixin, BaseConfigRule):
@@ -23,7 +23,7 @@ class FalconSecureFlagRule(SecureFlagRuleMixin, BaseConfigRule):
     def get_config_value(self, settings):
         return getattr(settings, self.SETTINGS_VALUE, None)
 
-    @fail_safely("Failed to apply config rule")
+    @fail_quietly("Failed to apply config rule")
     def apply(self, settings):
         """
         In falcon there isn't a way to get the config file so we set the default to the object containing the default config
