@@ -3,12 +3,12 @@
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import ast
 from inspect import getsourcefile
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 from contrast.utils.module_parser import get_ast_parsed_code
 
 
 class CodeParserMixin(object):
-    @fail_safely("Unable to ast parse module", log_level="debug")
+    @fail_quietly("Unable to ast parse module")
     def find_hardcoded_str(self, cls_or_module, attribute_name, code_ast):
         """
         Checks the code ast to determine if the attribute has been
@@ -45,7 +45,11 @@ class CodeParserMixin(object):
         hardcoded_value = []
 
         class AssignToStr(ast.NodeVisitor):
-            VALID_VALUE_TYPES = (ast.Str, ast.IfExp) + ((ast.Bytes,))
+            VALID_VALUE_TYPES = (
+                ast.Str,
+                ast.IfExp,
+                ast.Bytes,
+            )
 
             def visit_Assign(self, node):
                 """
