@@ -3,19 +3,21 @@
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import threading
 from contrast.agent import scope
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_loudly
 from contrast.utils import service_util
 from contrast.utils.timer import now_ms
 
 from contrast.extern import structlog as logging
 
 logger = logging.getLogger("contrast")
+HEARTBEAT_THREAD_NAME = "ContrastHeartbeat"
 
 
 class Heartbeat(threading.Thread):
     def __init__(self, settings):
         super().__init__()
         self.daemon = True
+        self.name = HEARTBEAT_THREAD_NAME
         self.settings = settings
         self.stopped = False
 
@@ -42,7 +44,7 @@ class Heartbeat(threading.Thread):
                 self.send_heartbeat()
                 service_util.sleep(self.get_sleep_time())
 
-    @fail_safely("Error sending a heartbeat message")
+    @fail_loudly("Error sending a heartbeat message")
     def send_heartbeat(self):
         from contrast.agent import service_client
 
@@ -50,8 +52,8 @@ class Heartbeat(threading.Thread):
 
         # only send after we have not updated after the last interval time set in the config
         if (
-            self.settings.last_update is not None
-            and (now_ms() - self.settings.last_update) > heartbeat_interval_ms
+            self.settings.last_update_service is not None
+            and (now_ms() - self.settings.last_update_service) > heartbeat_interval_ms
         ):
             if self.settings.config.is_service_bypassed:
                 from contrast.reporting import ReportingClient
