@@ -4,12 +4,18 @@
 from __future__ import print_function
 import platform
 import sys
-
+import threading
+import contrast
 from contrast import __version__, AGENT_CURR_WORKING_DIR
 from contrast.reporting import teamserver_messages
-from contrast.agent import patch_controller, service_client, scope
+from contrast.agent import (
+    patch_controller,
+    service_client,
+    scope,
+    thread_watcher,
+    agent_lib_manager,
+)
 from contrast.agent.request_context import RequestContext
-from contrast.agent.telemetry import telemetry_disabled
 from contrast.agent.assess.rules.response.xss import analyze_xss
 from contrast.agent.assess.rules.response.analyze import analyze_response_rules
 from contrast.agent.assess.preflight import update_preflight_hashes
@@ -23,16 +29,16 @@ from contrast.utils.decorators import cached_property
 from contrast.utils.exceptions.contrast_service_exception import (
     ContrastServiceException,
 )
-from contrast.utils.string_utils import truncate
 from contrast.utils.decorators import log_time_cm
 from contrast.utils.exceptions.security_exception import SecurityException
+from contrast.utils import inventory_utils
 from contrast.utils.library_reader.library_reader import LibraryReader
 from contrast.utils.loggers.logger import setup_agent_logger, setup_basic_agent_logger
 from contrast.utils.timer import now_ms_float, now_ms
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_loudly, fail_quietly
 from contrast.assess_extensions import cs_str
 
-if not telemetry_disabled():
+if not contrast.telemetry_disabled():
     from contrast.agent.telemetry import Telemetry
 else:
     Telemetry = None
@@ -96,9 +102,7 @@ class BaseMiddleware(RoutesMixin):
 
         self.settings = None
         self.routes = {}
-
-        if not self.initialize_settings(self.app_name):
-            logger.error("Unable to initialize Contrast Agent.")
+        if not self.initialize_settings():
             return
 
         if not self.settings.is_agent_config_enabled():
@@ -119,12 +123,15 @@ class BaseMiddleware(RoutesMixin):
         self.log_environment()
         self.warn_for_misleading_config()
 
-        self.reporting_client = (
-            ReportingClient() if self.settings.config.is_service_bypassed else None
-        )
+        self.reporting_client = ReportingClient()
+        self.reporting_client.start()
 
-        if self.reporting_client is not None:
-            self.reporting_client.start()
+        if self.settings.config.is_service_bypassed:
+            if not self.successful_startup_msgs():
+                logger.error(
+                    "Unable to initialize Contrast Agent.", direct_to_teamserver=1
+                )
+                return
 
         if not BaseMiddleware._loaded:
             self.initialize_libraries()
@@ -139,32 +146,56 @@ class BaseMiddleware(RoutesMixin):
 
             if not BaseMiddleware._loaded:
                 enable_providers()
-                self.scan_configs()
+                self.scan_configs_in_thread()
+
+        if (
+            self.settings.config.is_agent_lib_enabled
+            and self.settings.is_protect_enabled()
+        ):
+            agent_lib_manager.initialize()
+        else:
+            logger.debug("Not using agent-lib")
 
         self.log_finish_initialize()
-
         # Starting telemetry must happen at the very end of initialization
         # so we have all the data to send in startup messages.
         if not BaseMiddleware._loaded and Telemetry is not None:
-            self.telemetry = Telemetry()
-            self.telemetry.start()
+            contrast.TELEMETRY = Telemetry()
+            contrast.TELEMETRY.start()
 
         self.first_request = True
         self.request_start_time = None
         self.request_path = None
         BaseMiddleware._loaded = True
 
-    def initialize_settings(self, app_name):
+    @cached_property
+    def name(self):
+        raise NotImplementedError("Must implement name")
+
+    def successful_startup_msgs(self):
+        server_start_msg = teamserver_messages.ServerStart()
+        response = self.reporting_client.send_message(server_start_msg)
+        if response is None:
+            return False
+
+        server_start_msg.process_response(response, self.reporting_client)
+
+        app_start_msg = teamserver_messages.ApplicationStartup()
+        response = self.reporting_client.send_message(app_start_msg)
+        if response is None:
+            return False
+
+        app_start_msg.process_response(response, self.reporting_client)
+        return True
+
+    @fail_loudly("Unable to initialize Contrast Agent Settings.", return_value=False)
+    def initialize_settings(self):
         """
         Initialize agent settings.
 
         Returns True on settings being initialized and False if any failure.
         """
-        try:
-            self.settings = Settings(app_name=app_name)
-        except Exception as e:
-            logger.error(e)
-            return False
+        self.settings = Settings(app_name=self.app_name, framework_name=self.name)
         return True
 
     def initialize_service_client(self):
@@ -175,8 +206,9 @@ class BaseMiddleware(RoutesMixin):
         """
         try:
             service_client.send_startup_messages()
-        except (Exception, ContrastServiceException) as e:
-            logger.error(e)
+        except Exception as e:
+            # catches any exception including ContrastServiceException
+            logger.error(e, exc_info=e)
             return False
         return True
 
@@ -222,7 +254,7 @@ class BaseMiddleware(RoutesMixin):
             import pip
 
             logger.debug("Pip Version: %s", pip.__version__)
-        except:
+        except Exception:
             pass
 
         logger.debug(banner)
@@ -240,7 +272,7 @@ class BaseMiddleware(RoutesMixin):
         # this thread
         send_message_func = (
             self.reporting_client.add_message
-            if self.reporting_client
+            if self.settings.config.is_service_bypassed
             else service_client.send_messages
         )
 
@@ -274,10 +306,10 @@ class BaseMiddleware(RoutesMixin):
 
         return service_client.is_connected()
 
-    def call_with_agent(self, context, environ, start_response):
-        pass
+    def call_with_agent(self, *args):
+        raise NotImplementedError("Must implement call_with_agent")
 
-    def call_without_agent(self):
+    def call_without_agent(self, *args):
         """
         The agent does not set context when this function is called so all other patches
         (e.g propagators) that check context shouldn't run.
@@ -320,6 +352,8 @@ class BaseMiddleware(RoutesMixin):
             return
 
         with scope.contrast_scope():
+            thread_watcher.ensure_running(self)
+
             if self.settings.is_assess_enabled():
                 # route discovery and storage needs to occur before sending messages
                 self.handle_routes(context, request)
@@ -329,10 +363,16 @@ class BaseMiddleware(RoutesMixin):
 
             logger.debug("Sending final messages for reporting.")
 
+            context.truncate_request_body()
+
             if self.settings.config.is_service_bypassed:
                 for msg in self.final_ts_messages(context):
                     self.reporting_client.add_message(msg)
             else:
+                # messages now sent directly to TS
+                self.reporting_client.add_message(teamserver_messages.ServerActivity())
+
+                # messages still sent to SR until transition.
                 final_messages = self.final_messages(context)
                 service_client.send_messages(final_messages)
 
@@ -342,6 +382,7 @@ class BaseMiddleware(RoutesMixin):
         messages = [
             teamserver_messages.ServerActivity(),
             teamserver_messages.Preflight(context.activity),
+            teamserver_messages.Activity(context),
         ]
 
         # Currently we do not report an observed route if the route signature is empty.
@@ -359,14 +400,9 @@ class BaseMiddleware(RoutesMixin):
         return messages
 
     def final_messages(self, context):
-        # Request body is truncated to not overwhelm ContrastUI.
-        raw_request_body = context.activity.http_request.request_body_binary
-        context.activity.http_request.request_body_binary = truncate(
-            raw_request_body, length=4096
-        )
+        inventory_utils.append_db(context)
 
         final_messages = [
-            context.server_activity,
             context.activity,
         ]
 
@@ -380,7 +416,11 @@ class BaseMiddleware(RoutesMixin):
 
         if self.first_request and self.settings.is_inventory_enabled():
             # ApplicationUpdate message only sent at first request once we have routes
-            final_messages.append(service_client.build_update_message(self.routes))
+            final_messages.append(
+                service_client.build_update_message(
+                    self.routes, framework_name=self.name
+                )
+            )
 
         return final_messages
 
@@ -435,17 +475,10 @@ class BaseMiddleware(RoutesMixin):
         """
         Prefilter for any rules that do not yet use speedracer
         """
-        scope.enter_contrast_scope()
-        try:
+        with scope.contrast_scope():
             self.prefilter_defend()
-        except SecurityException:
-            logger.exception("PROTECT: threw security exception in prefilter")
-            raise
-        except:
-            logger.exception("PROTECT: threw exception in prefilter")
-        finally:
-            scope.exit_contrast_scope()
 
+    @fail_quietly("Failed to run prefilter protect.")
     def prefilter_defend(self):
         rules = self.settings.protect_rules
         logger.debug("PROTECT: Running Agent prefilter.")
@@ -462,13 +495,7 @@ class BaseMiddleware(RoutesMixin):
         """
         with log_time_cm("postfilter"):
             if self.settings.is_protect_enabled():
-                try:
-                    self.postfilter_defend()
-                except SecurityException as e:
-                    logger.exception("PROTECT: threw security exception in postfilter")
-                    raise e
-                except Exception:
-                    logger.exception("PROTECT: threw exception in postfilter")
+                self.postfilter_defend()
 
             if self.settings.is_assess_enabled():
                 with scope.contrast_scope():
@@ -508,7 +535,7 @@ class BaseMiddleware(RoutesMixin):
         """
         raise NotImplementedError("Children must define trigger_node property")
 
-    @fail_safely("Unable to do assess response analysis")
+    @fail_loudly("Unable to do assess response analysis")
     def response_analysis(self, context):
         """
         Run postfilter for any assess rules. Reflected xss rule runs by default.
@@ -525,9 +552,10 @@ class BaseMiddleware(RoutesMixin):
         if not self.settings.response_scanning_enabled:
             return
 
-        if self.reporting_client:
+        if self.settings.config.is_service_bypassed:
             analyze_response_rules(context, self.reporting_client)
 
+    @fail_quietly("Failed to run postfilter protect.")
     def postfilter_defend(self):
         rules = self.settings.protect_rules
         logger.debug("PROTECT: Running Agent postfilter.")
@@ -594,11 +622,17 @@ class BaseMiddleware(RoutesMixin):
             self.LOGS_SEPARATOR,
         )
 
-    def scan_configs(self):
+    def scan_configs_in_thread(self):
         """
-        Run config scanning rules for assess
+        Run config scanning rules for assess in a separate thread.
 
         Not all frameworks we support will necessarily have config scanning rules.
-        Those that do should override this method.
+        These frameworks should not start a thread.
         """
-        logger.debug(f"No config scanning rules for {self.__class__.__name__}")
+        if hasattr(self, "_scan_configs"):
+            logger.debug("Will start a thread to scan config rules.")
+            scanner = threading.Thread(target=self._scan_configs)
+            scanner.daemon = True
+            scanner.start()
+        else:
+            logger.debug(f"No config scanning rules for {self.__class__.__name__}")
