@@ -29,7 +29,7 @@ logger = logging.getLogger("contrast")
 
 @lru_cache(maxsize=1)
 def is_django_1():
-    settings = Settings()
+    settings = Settings(framework_name="django")
     return settings.framework.version.major == "1"
 
 
