@@ -2,11 +2,11 @@
 # Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from contrast.api import Route
-from contrast.agent.middlewares.route_coverage.coverage_utils import (
-    CoverageUtils,
+from contrast.agent.middlewares.route_coverage.common import (
     DEFAULT_ROUTE_METHODS,
+    get_normalized_uri,
+    build_key,
 )
-
 from contrast.agent.middlewares.route_coverage.common import build_route
 
 DEFAULT_ROUTE_METHODS = DEFAULT_ROUTE_METHODS + ("HEAD",)
@@ -27,10 +27,10 @@ def create_fastapi_routes(app):
         methods = app_route.methods or DEFAULT_ROUTE_METHODS
 
         for method_type in methods:
-            key = CoverageUtils.build_key(route_id, method_type)
+            key = build_key(route_id, method_type)
             routes[key] = Route(
                 verb=method_type,
-                url=CoverageUtils.get_normalized_uri(str(app_route.name)),
+                url=get_normalized_uri(str(app_route.name)),
                 route=route,
             )
 
