@@ -96,7 +96,7 @@ class RoutesMixin(object):
         # we can provide to customers. If a route doesn't show up in Contrast UI,
         # it may be due to its missing signature. In this scenario, we will have to work
         # with the customer directly to understand why the signature was not created.
-        if not route.route:
+        if not route.signature:
             logger.debug(
                 "No route signature found for %s : %s (id=%s). Not updating observed"
                 " route",
@@ -106,7 +106,7 @@ class RoutesMixin(object):
             )
             return
 
-        context.observed_route.signature = route.route
+        context.observed_route.signature = route.signature
 
         context.observed_route.url = route.url
         context.observed_route.verb = route.verb
