@@ -112,7 +112,8 @@ def enable_assess_patches(use_extension_hooks=True):
     string_propagation.build_string_propagator_functions()
 
     try:
-        module.hook = cs_str.initialize(logger)
+        module.hook = cs_str.initialize()
+
         cs_str.enable_required_hooks(module.hook)
     except RuntimeError:
         logger.error(
