@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2021 Contrast Security, Inc.
+# Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import os
 
-from contrast.extern.six.moves.urllib_parse import unquote
+from urllib.parse import unquote
 
 from contrast.api.dtm_pb2 import PathTraversalSemanticAnalysisDetails
 from contrast.utils.decorators import cached_property
