@@ -2,14 +2,13 @@
 # Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import inspect
-import os
 
 import contrast
 from contrast.agent.settings import Settings
 from contrast.api.dtm_pb2 import AttackResult, RaspRuleSample, UserInput
 from contrast.api.settings_pb2 import ProtectionRule
 from contrast.api.settings_pb2 import InputAnalysisResult
-from contrast.utils.decorators import fail_safely, fail_quietly
+from contrast.utils.decorators import fail_loudly, fail_quietly
 from contrast.utils.decorators import cached_property
 from contrast.utils.exceptions.security_exception import SecurityException
 from contrast.utils.stack_trace_utils import build_and_clean_stack
@@ -36,7 +35,7 @@ class BaseRule(object):
     Base rule object that all protection rules will inherit
     """
 
-    NAME = "base-rule"
+    RULE_NAME = "base-rule"
 
     def __init__(self):
         self.settings = Settings()
@@ -44,7 +43,7 @@ class BaseRule(object):
 
     @property
     def name(self):
-        return self.NAME
+        return self.RULE_NAME
 
     @property
     def mode(self):
@@ -78,7 +77,12 @@ class BaseRule(object):
         """
         Retrieve the mode based on the rule name
         """
-        for definition in self.settings.application_settings.protection_rules:
+        app_settings = (
+            self.settings.direct_ts_app_settings
+            if self.settings.config.is_service_bypassed
+            else self.settings.application_settings
+        )
+        for definition in app_settings.protection_rules:
             # "name" here is the "id" in the Settings DTM
             if definition.id == self.name:
                 return definition.mode
@@ -193,7 +197,7 @@ class BaseRule(object):
         if self.should_block(attack):
             raise SecurityException(self, f"Rule triggered. {match_string} blocked.")
 
-    @fail_safely("Failed to run protect rule")
+    @fail_loudly("Failed to run protect rule")
     def protect(self, patch_policy, user_input, args, kwargs):
         if not self.enabled:
             return
@@ -286,12 +290,11 @@ class BaseRule(object):
 
         attack = None
         for evaluation in evaluations_for_rule:
-            if analysis_stage == POSTFILTER:
-                if (
-                    evaluation.attack_count > 0
-                    or evaluation.input_type == UserInput.QUERYSTRING
-                ):
-                    continue
+            if analysis_stage == POSTFILTER and (
+                evaluation.attack_count > 0
+                or evaluation.input_type == UserInput.QUERYSTRING
+            ):
+                continue
 
             if candidate_string:
                 if candidate_string.find(evaluation.value) == -1:
@@ -394,7 +397,7 @@ class BaseRule(object):
         else:
             message = self.effective_attack_message()
 
-        logger.warning("%s %s", AttackResult.ResponseType.Name(response), message)
+        logger.debug("%s %s", AttackResult.ResponseType.Name(response), message)
 
     def log_rule_probed(self, evaluation):
         """
@@ -417,7 +420,7 @@ class BaseRule(object):
         else:
             message = self.effective_attack_message()
 
-        logger.info(message)
+        logger.debug(message)
 
     def effective_attack_message(self):
         return f"An effective attack was detected against {self.name}."
@@ -435,6 +438,7 @@ class BaseRule(object):
             return
 
         context.activity.results.extend([attack_result])
+        context.attacks.append(attack_result)
 
     _RESPONSE_MAP = {
         ProtectionRule.MONITOR: AttackResult.MONITORED,
@@ -445,14 +449,7 @@ class BaseRule(object):
     }
 
     def response_from_mode(self, mode):
-        response = self._RESPONSE_MAP.get(mode, None)
-
-        if response is None:
-            raise Exception(
-                f"Unable to convert Protect Mode to Attack Result {os.linesep} Unknown Mode: {mode}"
-            )
-
-        return response
+        return self._RESPONSE_MAP.get(mode)
 
     def evaluations_for_rule(self):
         context = contrast.CS__CONTEXT_TRACKER.current()
@@ -482,3 +479,10 @@ class BaseRule(object):
             method_name,
             ensure_string(user_input, errors="replace"),
         )
+
+    def sample_dtm_to_json(self, _):
+        """
+        Used to extract rule-specific details inside a RaspRuleSample
+        Should be implemented by each child class unless no details are known.
+        """
+        return {}
