@@ -20,7 +20,7 @@ class Deserialization(BaseRule):
     Deserialization Protection rule
     """
 
-    NAME = "untrusted-deserialization"
+    RULE_NAME = "untrusted-deserialization"
 
     # pickle and pyyaml both use load
     METHODS = [
@@ -195,10 +195,10 @@ class Deserialization(BaseRule):
         :return: InputAnalysisResult for this input
         """
         ia_result = InputAnalysisResult()
-        ia_result.rule_id = self.NAME
+        ia_result.rule_id = self.name
         ia_result.input_type = UserInput.UNKNOWN
         ia_result.value = value
-        ia_result.key = self.NAME
+        ia_result.key = self.name
         return ia_result
 
     def infilter_kwargs(self, user_input, patch_policy):
@@ -207,3 +207,9 @@ class Deserialization(BaseRule):
         return dict(
             deserializer=patch_policy.method_name, stack_elements=stack_elements
         )
+
+    def sample_dtm_to_json(self, sample):
+        return {
+            "command": sample.untrusted_deserialization.command,
+            "deserializer": sample.untrusted_deserialization.deserializer,
+        }
