@@ -10,7 +10,7 @@ class MalformedHeader(BaseRule):
     Malformed Header Protection rule
     """
 
-    NAME = "malformed-header"
+    RULE_NAME = "malformed-header"
 
     @property
     def mode(self):
