@@ -15,7 +15,7 @@ SAFE_PATHS = ["tmp", "public", "docs", "static", "template", "templates"]
 
 
 class PathTraversal(BaseRule, PathTraversalSinkFeatures):
-    NAME = "path-traversal"
+    RULE_NAME = "path-traversal"
 
     def find_attack(self, candidate_string=None, analysis_stage=None, **kwargs):
         """
@@ -49,6 +49,9 @@ class PathTraversal(BaseRule, PathTraversalSinkFeatures):
     def infilter_kwargs(self, user_input, patch_policy):
         return dict(method=patch_policy.method_name)
 
+    def sample_dtm_to_json(self, sample):
+        return {"path": sample.path_traversal.path}
+
 
 def possible_write(args, kwargs):
     if "w" in kwargs.get("mode", ""):
