@@ -25,20 +25,20 @@ def build_protect_rules():
     """
     rules = OrderedDict(
         {
-            UnsafeFileUpload.NAME: UnsafeFileUpload(),
-            CmdInjection.NAME: CmdInjection(),
-            Deserialization.NAME: Deserialization(),
+            UnsafeFileUpload.RULE_NAME: UnsafeFileUpload(),
+            CmdInjection.RULE_NAME: CmdInjection(),
+            Deserialization.RULE_NAME: Deserialization(),
             # Turned off until TS can handle rule information
-            MalformedHeader.NAME: MalformedHeader(),
-            MethodTampering.NAME: MethodTampering(),
-            NoSqlInjection.NAME: NoSqlInjection(),
+            MalformedHeader.RULE_NAME: MalformedHeader(),
+            MethodTampering.RULE_NAME: MethodTampering(),
+            NoSqlInjection.RULE_NAME: NoSqlInjection(),
             # Padding Oracle rule is currently disabled - CONTRAST-35352
-            # PaddingOracle.NAME: PaddingOracle(),
-            PathTraversal.NAME: PathTraversal(),
-            SqlInjection.NAME: SqlInjection(),
-            Ssrf.NAME: Ssrf(),
-            Xss.NAME: Xss(),
-            Xxe.NAME: Xxe(),
+            # PaddingOracle.RULE_NAME: PaddingOracle(),
+            PathTraversal.RULE_NAME: PathTraversal(),
+            SqlInjection.RULE_NAME: SqlInjection(),
+            Ssrf.RULE_NAME: Ssrf(),
+            Xss.RULE_NAME: Xss(),
+            Xxe.RULE_NAME: Xxe(),
         }
     )
 
