@@ -22,7 +22,7 @@ class SqlInjection(BaseRule):
     SQL Injection Protection rule
     """
 
-    NAME = "sql-injection"
+    RULE_NAME = "sql-injection"
 
     def build_attack_with_match(
         self, candidate_string, evaluation=None, attack=None, **kwargs
@@ -90,6 +90,15 @@ class SqlInjection(BaseRule):
     def infilter_kwargs(self, user_input, patch_policy):
         return dict(database=patch_policy.module)
 
+    def sample_dtm_to_json(self, sample):
+        return {
+            "start": sample.sqli.start_idx,
+            "end": sample.sqli.end_idx,
+            "boundaryOverrunIndex": sample.sqli.boundary_overrun_idx,
+            "inputBoundaryIndex": sample.sqli.input_boundary_idx,
+            "query": sample.sqli.query,
+        }
+
     def skip_protect_analysis(self, user_input, args, kwargs):
         """
         Some sql libraries use special objects (see from sqlalchemy import text)
