@@ -14,7 +14,7 @@ class Ssrf(BaseRule):
     Currently in BETA.
     """
 
-    NAME = "ssrf"
+    RULE_NAME = "ssrf"
 
     def is_postfilter(self):
         return False
@@ -24,3 +24,6 @@ class Ssrf(BaseRule):
         if url is not None:
             sample.ssrf.url = url
         return sample
+
+    def sample_dtm_to_json(self, sample):
+        return {"url": sample.ssrf.url}
