@@ -10,4 +10,4 @@ class UnsafeFileUpload(BaseRule):
     files that get uploaded
     """
 
-    NAME = "unsafe-file-upload"
+    RULE_NAME = "unsafe-file-upload"
