@@ -11,7 +11,7 @@ class Xss(BaseRule):
     Currently only a prefilter / block at perimeter rule
     """
 
-    NAME = "reflected-xss"
+    RULE_NAME = "reflected-xss"
 
     @property
     def mode(self):
@@ -25,3 +25,16 @@ class Xss(BaseRule):
             if mode in [ProtectionRule.NO_ACTION, ProtectionRule.MONITOR]
             else ProtectionRule.BLOCK_AT_PERIMETER
         )
+
+    def sample_dtm_to_json(self, sample):
+        return {
+            "input": sample.xss.input,
+            "matches": [
+                {
+                    "evidenceStart": match.evidence_start_ms,
+                    "evidence": match.evidence,
+                    "offset": match.offset,
+                }
+                for match in sample.xss.matches
+            ],
+        }
