@@ -14,7 +14,7 @@ class Xxe(BaseRule):
     XXE Protection rule
     """
 
-    NAME = "xxe"
+    RULE_NAME = "xxe"
     INPUT_NAME = "XML Prolog"
 
     EXTERNAL_ENTITY_PATTERN = re.compile(
@@ -84,7 +84,7 @@ class Xxe(BaseRule):
         sample.xxe.xml = self.prolog_xml
         return sample
 
-    def build_base_sample(self, evaluation):
+    def build_base_sample(self, evaluation, prebuilt_stack=None):
         sample = RaspRuleSample()
         sample.stack_trace_elements.extend(build_and_clean_stack())
 
@@ -101,3 +101,16 @@ class Xxe(BaseRule):
 
     def infilter_kwargs(self, user_input, patch_policy):
         return dict(framework=patch_policy.method_name)
+
+    def sample_dtm_to_json(self, sample):
+        return {
+            "xml": sample.xxe.xml,
+            "declaredEntities": [
+                {"start": entity.start_idx, "end": entity.end_idx}
+                for entity in sample.xxe.declared_entities
+            ],
+            "entitiesResolved": [
+                {"systemId": entity.system_id, "publicId": entity.public_id}
+                for entity in sample.xxe.entities_resolved
+            ],
+        }
