@@ -7,7 +7,9 @@ from contrast.extern import webob
 
 from contrast.api import dtm_pb2
 from contrast.utils.timer import now_ms
-from contrast.agent.middlewares.route_coverage.coverage_utils import CoverageUtils
+from contrast.agent.middlewares.route_coverage.common import (
+    get_normalized_uri as strip_uri,
+)
 
 from contrast.extern import structlog as logging
 
@@ -84,7 +86,7 @@ class Request(webob.BaseRequest):
         if self._normalized_uri is not None:
             return self._normalized_uri
 
-        self._normalized_uri = CoverageUtils.get_normalized_uri(self.path)
+        self._normalized_uri = strip_uri(self.path)
         return self._normalized_uri
 
     def get_body(self, as_text=False, errors="ignore"):
