@@ -5,12 +5,12 @@ from contrast.agent import scope
 
 from contrast.agent.request import Request
 from contrast.agent.settings import Settings
-from contrast.api.dtm_pb2 import Activity, ServerActivity, ObservedRoute
+from contrast.api.dtm_pb2 import Activity, ObservedRoute
 
 from contrast.extern import structlog as logging
 from contrast.utils.digest_utils import Digest
 from contrast.utils.decorators import cached_property, fail_quietly
-from contrast.utils.string_utils import ensure_binary
+from contrast.utils.string_utils import ensure_binary, truncate
 
 logger = logging.getLogger("contrast")
 
@@ -25,7 +25,8 @@ class RequestContext(object):
         self.activity = Activity()
         self.activity.http_request.CopyFrom(dtm)
 
-        self.server_activity = ServerActivity()
+        # For protect: store attacks made during a request to report at the end
+        self.attacks = []
 
         self.speedracer_input_analysis = None
         self.do_not_track = False
@@ -34,7 +35,7 @@ class RequestContext(object):
         self.current_route = None
 
         self.observed_route = ObservedRoute()
-
+        self.database_info = []
         self.source_count = 0
         self.propagation_count = 0
 
@@ -156,3 +157,10 @@ class RequestContext(object):
             for finding in self.activity.findings
             if finding.rule_id == "reflected-xss"
         ]
+
+    def truncate_request_body(self):
+        # Request body is truncated to not overwhelm ContrastUI.
+        raw_request_body = self.activity.http_request.request_body_binary
+        self.activity.http_request.request_body_binary = truncate(
+            raw_request_body, length=4096
+        )
