@@ -18,7 +18,6 @@ from contrast.api.dtm_pb2 import (
     Message,
     ObservedRoute,
     Poll,
-    ServerActivity,
 )
 from contrast.api.settings_pb2 import AgentSettings
 from contrast.utils.exceptions.contrast_service_exception import (
@@ -351,9 +350,6 @@ class _ServiceState(Singleton):
         if isinstance(agent_message, AgentStartup):
             message.agent_startup.CopyFrom(agent_message)
 
-        if isinstance(agent_message, ServerActivity):
-            message.server_activity.CopyFrom(agent_message)
-
         if isinstance(agent_message, ApplicationCreate):
             message.application_create.CopyFrom(agent_message)
 
@@ -375,7 +371,6 @@ class _ServiceState(Singleton):
         """
         Possible messages supported currently
 
-        contrast.api.dtm.ServerActivity server_activity = 10;
         contrast.api.dtm.AgentStartup agent_startup = 11;
         contrast.api.dtm.ApplicationCreate application_create = 12;
         contrast.api.dtm.ApplicationUpdate application_update = 13;
@@ -430,7 +425,7 @@ class _ServiceState(Singleton):
         create.metadata = settings.config.get("application.metadata", "")
 
         # build based views
-        create.session_id = settings.config.get_session_id()
+        create.session_id = settings.config.session_id
         create.session_metadata = settings.config.get_session_metadata()
 
         # instrumentation mode
@@ -444,14 +439,14 @@ def is_connected():
     return _ServiceState().connection_status.connected
 
 
-def build_update_message(routes):
+def build_update_message(routes, framework_name):
     """
     Create an ApplicationUpdate message with framework information and routes.
 
     :param routes: Dict of api.Route, will be empty if in Protect mode
     :return: ApplicationUpdate instance
     """
-    settings = Settings()
+    settings = Settings(framework_name=framework_name)
 
     update_message = ApplicationUpdate()
 
