@@ -15,9 +15,10 @@ from contrast.api.settings_pb2 import (
     AgentSettings as DtmAgentSettings,
     ApplicationSettings as DtmApplicationSettings,
     ServerFeatures as DtmServerFeatures,
+    ProtectionRule as DtmProtectionRule,
 )
-from contrast.configuration import AgentConfig
-from contrast.utils.decorators import cached_property, fail_safely
+from contrast.configuration import AgentConfig, SamplingLockConfig
+from contrast.utils.decorators import cached_property, fail_loudly
 from contrast.utils.loggers.logger import reset_agent_logger
 from contrast.utils.singleton import Singleton
 from contrast.utils.string_utils import truncate
@@ -26,6 +27,8 @@ from contrast.extern import structlog as logging
 
 logger = logging.getLogger("contrast")
 
+ASSESS_STACKTRACES = "assess.stacktraces"
+
 
 class ServerFeatures:
     def __init__(self, features):
@@ -52,8 +55,64 @@ class ServerFeatures:
         return self.features.get("features", {}).get("defend", {}).get("enabled", False)
 
 
+class ApplicationSettings:
+    def __init__(self, settings_json):
+        self._raw_settings = settings_json or {}
+
+    @cached_property
+    def disabled_assess_rules(self):
+        return (
+            self._raw_settings.get("settings", {})
+            .get("assessment", {})
+            .get("disabledRules", [])
+        )
+
+    @cached_property
+    def protection_rules(self):
+        return [
+            ProtectRule(r)
+            for r in self._raw_settings.get("settings", {})
+            .get("defend", {})
+            .get("protectionRules", [])
+        ]
+
+    @cached_property
+    def session_id(self):
+        return (
+            self._raw_settings.get("settings", {})
+            .get("assessment", {})
+            .get("session_id")
+        )
+
+    # note: there are many more fields on the ApplicationSettings that we currently do
+    # not use
+
+
+class ProtectRule:
+    def __init__(self, protect_rule_json):
+        self._raw_rule = protect_rule_json or {}
+
+    @cached_property
+    def id(self):
+        return self._raw_rule.get("id", "")
+
+    @cached_property
+    def mode(self):
+        raw_mode = self._raw_rule.get("mode", "OFF")
+        raw_block_at_entry = self._raw_rule.get("blockAtEntry", False)
+        return {
+            "MONITORING": DtmProtectionRule.MONITOR,
+            "BLOCKING": (
+                DtmProtectionRule.BLOCK_AT_PERIMETER
+                if raw_block_at_entry
+                else DtmProtectionRule.BLOCK
+            ),
+            "OFF": DtmProtectionRule.NO_ACTION,
+        }[raw_mode]
+
+
 class Settings(Singleton, SettingsREPMixin):
-    def init(self, app_name=None):
+    def init(self, app_name=None, framework_name=None):
         """
         Agent settings for the entire lifetime of the agent.
 
@@ -61,10 +120,11 @@ class Settings(Singleton, SettingsREPMixin):
         """
         self.config = None
         self.config_features = {}
-        self.last_update = None
-        self.last_update_time_ms = 0
+        self.last_update_service = None
+        self.last_server_update_time_ms = 0
+        self.last_app_update_time_ms = 0
         self.heartbeat = None
-        self.framework = Framework()
+        self.framework = Framework(framework_name)
         self.server = Server()
         self.sys_module_count = 0
         self.rewriter_enabled = False
@@ -92,10 +152,42 @@ class Settings(Singleton, SettingsREPMixin):
         from contrast.agent.assess.rules.response.cache_controls_rule import (
             CacheControlsRule,
         )
+        from contrast.agent.assess.rules.response.clickjacking_rule import (
+            ClickjackingRule,
+        )
+
+        from contrast.agent.assess.rules.response.x_content_type_rule import (
+            XContentTypeRule,
+        )
+
+        from contrast.agent.assess.rules.response.csp_header_missing_rule import (
+            CspHeaderMissingRule,
+        )
+
+        from contrast.agent.assess.rules.response.x_xss_protection_disabled_rule import (
+            XXssProtectionDisabledRule,
+        )
+
+        from contrast.agent.assess.rules.response.hsts_header_rule import HstsHeaderRule
+
+        from contrast.agent.assess.rules.response.csp_header_insecure_rule import (
+            CspHeaderInsecureRule,
+        )
+
+        from contrast.agent.assess.rules.response.parameter_pollution_rule import (
+            ParameterPollutionRule,
+        )
 
         self.assess_response_rules = [
             AutocompleteMissingRule(),
             CacheControlsRule(),
+            ClickjackingRule(),
+            XContentTypeRule(),
+            CspHeaderMissingRule(),
+            XXssProtectionDisabledRule(),
+            HstsHeaderRule(),
+            CspHeaderInsecureRule(),
+            ParameterPollutionRule(),
         ]
 
         # Initialize config
@@ -103,6 +195,7 @@ class Settings(Singleton, SettingsREPMixin):
 
         if self.config.is_service_bypassed:
             self.direct_teamserver_server_features = ServerFeatures(None)
+            self.direct_ts_app_settings = ApplicationSettings(None)
 
         self.disabled_assess_rules = set(
             self.config.get("assess.rules.disabled_rules", [])
@@ -247,7 +340,7 @@ class Settings(Singleton, SettingsREPMixin):
         reload_rules = False
 
         if data and isinstance(data, DtmAgentSettings):
-            self.last_update = data.sent_ms
+            self.last_update_service = data.sent_ms
 
             reload_rules = self.process_server_features(data) or reload_rules
             reload_rules = self.process_application_settings(data) or reload_rules
@@ -262,7 +355,13 @@ class Settings(Singleton, SettingsREPMixin):
 
     def apply_ts_server_settings(self, response_body):
         self.direct_teamserver_server_features = ServerFeatures(response_body)
-        self.last_update_time_ms = now_ms() - self.last_update_time_ms
+
+        if (
+            self.direct_teamserver_server_features.features.get("features", {})
+            .get("assessment", {})
+            .get("sampling")
+        ):
+            self.update_sampling()
 
         self.update_logger_from_features()
 
@@ -270,6 +369,21 @@ class Settings(Singleton, SettingsREPMixin):
 
         self.set_protect_rules()
 
+        self.last_server_update_time_ms = now_ms()
+
+    def process_ts_reactions(self, response_body):
+        # App startup/activity wrap reactions in a settings dict whereas
+        # Server startup/activity has it at the top level response dict
+        reactions = response_body.get("settings", {}).get("reactions", None)
+
+        if not reactions:
+            reactions = response_body.get("reactions", None)
+
+        if not reactions:
+            return
+
+        ReactionProcessor.process(reactions, self)
+
     def process_server_features(self, data):
         if not data.HasField("server_features"):
             return False
@@ -277,6 +391,9 @@ class Settings(Singleton, SettingsREPMixin):
         self.server_features = data.server_features
         self.update_logger_from_features()
 
+        if not self.config.is_service_bypassed and self.server_features.assess.enabled:
+            self.update_sampling()
+
         self.log_server_features(data.server_features)
 
         return True
@@ -310,6 +427,14 @@ class Settings(Singleton, SettingsREPMixin):
     def code_exclusion_matchers(self):
         return [x for x in self.exclusion_matchers if x.is_code]
 
+    def apply_ts_app_settings(self, response_json):
+        self.direct_ts_app_settings = ApplicationSettings(response_json)
+
+        # This is the only place session_id is set by TS.
+        # If session id is set in the config, that value will be echoed back by TS
+        if self.direct_ts_app_settings.session_id:
+            self.config.session_id = self.direct_ts_app_settings.session_id
+
     def process_application_settings(self, data):
         if not data.HasField("application_settings"):
             return False
@@ -455,10 +580,15 @@ class Settings(Singleton, SettingsREPMixin):
         """
         Rules disabled in config override all disabled rules from TS per common config
         """
+        app_settings = (
+            self.direct_ts_app_settings
+            if self.config.is_service_bypassed
+            else self.application_settings
+        )
         return (
             rule_id in self.disabled_assess_rules
             if self.disabled_assess_rules
-            else rule_id in self.application_settings.disabled_assess_rules
+            else rule_id in app_settings.disabled_assess_rules
         )
 
     def enabled_response_rules(self):
@@ -469,19 +599,13 @@ class Settings(Singleton, SettingsREPMixin):
         ]
 
     def is_collect_stacktraces_all(self):
-        return (
-            self.config is not None and self.config.get("assess.stacktraces") == "ALL"
-        )
+        return self.config is not None and self.config.get(ASSESS_STACKTRACES) == "ALL"
 
     def is_collect_stacktraces_some(self):
-        return (
-            self.config is not None and self.config.get("assess.stacktraces") == "SOME"
-        )
+        return self.config is not None and self.config.get(ASSESS_STACKTRACES) == "SOME"
 
     def is_collect_stacktraces_none(self):
-        return (
-            self.config is not None and self.config.get("assess.stacktraces") == "NONE"
-        )
+        return self.config is not None and self.config.get(ASSESS_STACKTRACES) == "NONE"
 
     def build_proxy_url(self):
         if self.proxy_url:
@@ -489,7 +613,7 @@ class Settings(Singleton, SettingsREPMixin):
 
         return {}
 
-    @fail_safely("Unable to ignore request", return_value=False)
+    @fail_loudly("Unable to ignore request", return_value=False)
     def ignore_request(self, path):
         """
         Determine if the given path exactly matches any of the
@@ -507,3 +631,51 @@ class Settings(Singleton, SettingsREPMixin):
                 logger.debug("Path %s matched on pattern %s", path, pattern)
                 return True
         return False
+
+    def update_sampling(self):
+        server_sampling = (
+            self.direct_teamserver_server_features.features.get("features")
+            .get("assessment")
+            .get("sampling")
+            if self.config.is_service_bypassed
+            else self.server_features.assess.sampling
+        )
+
+        sampling_lock_config = SamplingLockConfig()
+
+        if not sampling_lock_config.lock_enable:
+            self.config.put(
+                "assess.sampling.enable",
+                server_sampling.get("enable")
+                if self.config.is_service_bypassed
+                else server_sampling.enabled,
+            )
+
+        if not sampling_lock_config.lock_baseline:
+            self.config.put(
+                "assess.sampling.baseline",
+                server_sampling.get("baseline")
+                if self.config.is_service_bypassed
+                else server_sampling.baseline,
+            )
+
+        if not sampling_lock_config.lock_frequency:
+            self.config.put(
+                "assess.sampling.request_frequency",
+                server_sampling.get("frequency")
+                if self.config.is_service_bypassed
+                else server_sampling.request_frequency,
+            )
+
+        if not sampling_lock_config.lock_window:
+            self.config.put(
+                "assess.sampling.window_ms",
+                server_sampling.get("window")
+                if self.config.is_service_bypassed
+                else server_sampling.window_ms,
+            )
+
+        logger.debug(
+            "Updated sampling setting with TeamServer sampling settings: %s",
+            server_sampling,
+        )
