@@ -1,16 +1,14 @@
 # -*- coding: utf-8 -*-
 # Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
-from datetime import datetime, timezone
+from collections import defaultdict
 import os
 import uuid
 import hashlib
-import platform
-import re
-import sys
 from pathlib import Path
 from queue import Queue
 import threading
+import contrast
 
 from contrast import __version__
 from contrast.agent import scope
@@ -19,19 +17,15 @@ from contrast.agent.settings import Settings
 from contrast.extern import structlog as logging
 from requests import post as post_request, get as get_request
 from contrast.utils import service_util
-from contrast.utils.singleton import Singleton
 from contrast.utils.decorators import cached_property
-
-
-def telemetry_disabled():
-    OPT_OUT = os.environ.get("CONTRAST_AGENT_TELEMETRY_OPTOUT")
-    return OPT_OUT and OPT_OUT.lower() in ("1", "true")
+from contrast.agent.events import StartupMetricsTelemetryEvent, ErrorTelemetryEvent
 
 
 logger = logging.getLogger("contrast")
 
 FILE_LOCATIONS = [
-    "/etc/contrast/python/",  # Most stable place but some OS may not allow creating here
+    # Most stable place but some OS may not allow creating here
+    "/etc/contrast/python/",
     os.path.join(AGENT_CURR_WORKING_DIR, "config", "contrast"),
 ]
 
@@ -47,18 +41,24 @@ DISCLAIMER = (
 )
 
 BASE_URL = "https://telemetry.python.contrastsecurity.com"
-ENDPOINT = "api/v1/telemetry/metrics"
-URL = f"{BASE_URL}/{ENDPOINT}"
+BASE_ENDPOINT = "api/v1/telemetry"
+URL = f"{BASE_URL}/{BASE_ENDPOINT}"
 HEADERS = {"User-Agent": f"python-{__version__}"}
 
+TELEMETRY_THREAD_NAME = "ContrastTelemetry"
 
-class Telemetry(Singleton, threading.Thread):
-    SLEEP = 10800  # 3hrs
+
+class Telemetry(threading.Thread):
+    # While the telemetry spec suggests sleeping for 3 hours, we've decided
+    # that 30 minutes is reasonable for this agent.
+    SLEEP = 60 * 30  # 30 mins
     RETRY_SLEEP = 60
+    REPORTED_ERRORS = {}
 
-    def init(self):
+    def __init__(self):
         super().__init__()
         self.daemon = True
+        self.name = TELEMETRY_THREAD_NAME
 
         self.enabled = True
         self.message_q = None
@@ -111,13 +111,13 @@ class Telemetry(Singleton, threading.Thread):
             # Do not move creating startup msg outside of this function
             # so the work stays in the telemetry thread, not the main thread.
             self.add_message(StartupMetricsTelemetryEvent())
+            if not self.stopped:
+                self.send_messages()
 
             # This while loop should complete, shutting down the thread
             # if agent has become disabled during the course of the agent lifecycle.
             while not self.stopped and self.settings.is_agent_config_enabled():
-                if not self.message_q.empty():
-                    self.send_message()
-
+                self.send_messages()
                 service_util.sleep(self.wait_time)
 
     def add_message(self, msg):
@@ -127,21 +127,87 @@ class Telemetry(Singleton, threading.Thread):
         logger.debug("Adding msg to telemetry queue: %s", msg)
         self.message_q.put(msg)
 
-    def send_message(self):
-        msg = self.message_q.get()
+    def send_messages(self):
+        """
+        Send all messages in the queue, one batch request for each path for group of
+        messages.
+        """
+        all_messages = self.get_all_messages_by_path()
 
-        try:
-            logger.debug("Sending Telemetry msg %s", msg)
-            response = self._post(msg)
-            self._check_response(response)
-        except Exception as ex:
-            logger.error("Could not send message from telemetry queue.", ex)
+        for path, messages in all_messages.items():
+            try:
+                response = self._post(messages, path)
+                self._check_response(response)
+            except Exception as ex:
+                logger.debug("Could not send batch of telemetry messages.", ex)
+
+    def get_all_messages_by_path(self):
+        """
+        Return a dict of key path of message => list of messages for that path.
+
+        :return: dict
+        """
+        msgs = defaultdict(list)
+
+        while True:
+            msg = self.get_msg_blocking()  # will block thread until msg is available.
+            msgs[msg.path].append(msg)
+            if self.message_q.empty():
+                break
+        return msgs
+
+    def get_msg_blocking(self):
+        """
+        Our preferred way to pop a message off the queue is to
+        rely on .get()'s blocking mechanism.
+        """
+        logger.debug("Will wait until msg is available ...")
+        msg = self.message_q.get()  # will block thread until msg is available.
+        logger.debug("Got msg %s", msg)
+        return msg
+
+    def should_report_error(self, error, original_func):
+        key = " ".join(
+            [
+                type(error).__name__,
+                original_func.__name__,
+                original_func.__module__,
+                str(error),
+            ]
+        )
+
+        if key in self.REPORTED_ERRORS:
+            self.REPORTED_ERRORS[key] += 1
+            return False
+
+        self.REPORTED_ERRORS[key] = 1
+
+        return True
+
+    def report_error(self, error, original_func, logger_="", message=""):
+        """
+        Report an agent error/exception to Telemetry.
+
+        Take great care to avoid calling this where application errors or customer
+        code may be caught.
+        """
+        if self.should_report_error(error, original_func):
+            self.add_message(
+                ErrorTelemetryEvent(error=error, logger_=logger_, message=message)
+            )
+
+    def _post(self, messages, path):
+        """
+        Send a list of `messages` to telemetry `path`
+        """
+        logger.debug("Sending %s Telemetry messages to %s.", len(messages), path)
 
-    def _post(self, msg):
-        data = [msg.to_json()]
-        url = f"{URL}{msg.path}"
         response = post_request(
-            url, json=data, headers=HEADERS, allow_redirects=False, verify=True
+            f"{URL}{path}",
+            json=[msg.to_json() for msg in messages],
+            headers=HEADERS,
+            allow_redirects=False,
+            verify=True,
         )
 
         logger.debug("Telemetry response: %s %s", response.status_code, response.reason)
@@ -175,7 +241,7 @@ class Telemetry(Singleton, threading.Thread):
         return hashlib.sha256(str_input.encode()).hexdigest()
 
     def _check_enabled(self):
-        if telemetry_disabled() or self._connection_failed():
+        if contrast.telemetry_disabled() or self._connection_failed():
             self.enabled = False
         else:
             self._find_or_create_file()
@@ -245,126 +311,3 @@ class Telemetry(Singleton, threading.Thread):
         # be created
         print(DISCLAIMER)  # pylint: disable=superfluous-parens
         logger.info(DISCLAIMER)
-
-
-class MetricsDict(dict):
-    def __init__(self, value_type):
-        self._value_type = value_type
-
-    def __setitem__(self, key, value):
-        if not isinstance(key, str):
-            logger.debug(
-                "WARNING: non-string key will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-            )
-            return
-        if not re.fullmatch(r"[a-zA-Z0-9\._-]{1,63}", key):
-            logger.debug(
-                "WARNING: invalid key will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-            )
-            return
-        if len(key) >= 28:
-            # we enforce this condition separately from the regex in order to exactly
-            # align with regex in the spec
-            logger.debug(
-                "WARNING: too-long key will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-            )
-            return
-        if not isinstance(value, self._value_type):
-            logger.debug(
-                "WARNING: wrong-type value will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-                expected_type=self._value_type,
-            )
-            return
-        if self._value_type == str and len(value) == 0:
-            logger.debug(
-                "WARNING: blank string value will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-                expected_type=self._value_type,
-            )
-            return
-        if self._value_type == str and len(value) > 200:
-            logger.debug(
-                "WARNING: too-long string value will be omitted from telemetry metrics",
-                key=key,
-                value=value,
-            )
-            return
-
-        key = key.lower()
-        super().__setitem__(key, value)
-
-
-class TelemetryEvent(object):
-    def __init__(self):
-        self.timestamp = datetime.now(timezone.utc).astimezone().isoformat()
-        self.telemetry_instance_id = Telemetry().instance_id
-        self.tags = MetricsDict(str)
-
-    @property
-    def path(self):
-        return ""
-
-    def to_json(self):
-        return dict(
-            timestamp=self.timestamp,
-            instance=self.telemetry_instance_id,
-            tags=self.tags,
-        )
-
-
-class MetricsTelemetryEvent(TelemetryEvent):
-    def __init__(self):
-        super().__init__()
-        self.fields = MetricsDict(int)
-        self.fields["_filler"] = 0
-
-    def __repr__(self):
-        return f"{self.__class__.__name__}: {self.to_json()}"
-
-    def to_json(self):
-        return {
-            **super().to_json(),
-            **dict(fields=self.fields),
-        }
-
-
-class StartupMetricsTelemetryEvent(MetricsTelemetryEvent):
-    def __init__(self):
-        super().__init__()
-
-        try:
-            self.add_tags()
-        except Exception as ex:
-            logger.debug("Telemetry failed to create StartupMetrics msg: %s", ex)
-            logger.debug("Disabling telemetry")
-            Telemetry().enabled = False
-
-    @property
-    def path(self):
-        return "/startup"
-
-    def add_tags(self):
-        telemetry = Telemetry()
-        settings = telemetry.settings
-
-        self.tags["is_public_build"] = str(telemetry.is_public_build).lower()
-        self.tags["agent_version"] = __version__
-        self.tags["python_version"] = sys.version
-        self.tags["os_type"] = platform.system()
-        self.tags["os_arch"] = platform.machine()
-        self.tags["os_version"] = platform.version()
-        self.tags["app_framework_version"] = str(settings.framework)
-        self.tags["server_framework_version"] = str(settings.server)
-        self.tags["teamserver"] = settings.config.teamserver_type
-
-        if settings.rewriter_enabled:
-            self.tags["rewriter_enabled"] = "true"
