@@ -2,7 +2,6 @@
 # Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 from contrast.api.dtm_pb2 import Activity, Finding as FindingDtm
-
 from contrast.utils.string_utils import protobuf_safe
 
 from contrast.agent.policy.constants import (
@@ -70,18 +69,31 @@ def send_finding(finding, context=None):
     Send a finding by either appending it to request context
     OR sending it immediately.
 
+    If `context` exists, agent should not sent message immediately because
+    current route needs to be appended in `append_route_to_findings`
+
     :param finding:  api.dtm_pb2.Finding instance
     :param context: Request context instance
     :return: None
     """
-    # Import here to prevent circular import.
-    from contrast.agent.service_client import send_messages
-
     if context:
-        # Once RequestContext is deprecated, this flow can be removed.
         context.activity.findings.extend([finding])
-    else:
-        activity = Activity()
-        activity.findings.extend([finding])
+        return
 
+    activity = Activity()
+    activity.findings.extend([finding])
+
+    from contrast.agent.settings import Settings
+    settings = Settings()
+
+    if settings.config.is_service_bypassed:
+        from contrast.reporting import ReportingClient
+        from contrast.reporting import teamserver_messages
+
+        client = ReportingClient()
+        client.add_message(teamserver_messages.Preflight(activity))
+
+    else:
+        # Import here to prevent circular import.
+        from contrast.agent.service_client import send_messages
         send_messages([activity])
