@@ -15,18 +15,18 @@ class Route(object):
     def __init__(self, verb, url, route):
         self.verb = verb or "GET"
         self.url = url or ""
-        self.route = route or ""
+        self.signature = route or ""
 
     def to_dtm(self):
         route = RouteDtm()
         route.verb = self.verb
         route.url = self.url
-        route.route = self.route
+        route.route = self.signature
         return route
 
     def to_json(self):
         return {
-            "signature": self.route,
+            "signature": self.signature,
             "verb": self.verb,
             "url": self.url,
         }
