@@ -24,7 +24,7 @@ def apply_rule(patch_policy, orig_func, args, kwargs):
     ):
         for node in patch_policy.trigger_nodes:
             rule_name = node.rule.name
-            rule = Settings().defend_rules.get(rule_name)
+            rule = Settings().protect_rules.get(rule_name)
             if not rule or not rule.enabled:
                 continue
 
