@@ -11,10 +11,10 @@ from contrast.agent.policy.loader import Policy
 from contrast.agent.assess.policy.analysis import skip_analysis
 from contrast.agent.assess.policy import trigger_policy
 from contrast.agent.settings import Settings
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 
 
-@fail_safely("Error running SQLi assess rule")
+@fail_quietly("Error running SQLi assess rule")
 def assess_rule(context, adapter_name, method, result, args, kwargs):
     """
     Apply assess SQLi rule
@@ -67,7 +67,7 @@ def apply_rule(adapter_name, orig_func, args, kwargs):
     result = None
 
     if context is not None:
-        _protect_rule(context, adapter_name, orig_func.__name__, args, kwargs)
+        _protect_rule(adapter_name, orig_func.__name__, args, kwargs)
 
     try:
         result = orig_func(*args, **kwargs)
@@ -78,8 +78,8 @@ def apply_rule(adapter_name, orig_func, args, kwargs):
     return result
 
 
-@fail_safely("Error running SQLi protect rule")
-def _protect_rule(context, adapter_name, method, args, kwargs):
+@fail_quietly("Error running SQLi protect rule")
+def _protect_rule(adapter_name, method, args, kwargs):
     """
     Run protect SQLi analysis if it's enabled.
     """
@@ -89,12 +89,15 @@ def _protect_rule(context, adapter_name, method, args, kwargs):
     # args[0] is self, args[1] is the query
     sql = args[1]
 
-    # TODO: PYT-986: remove creating fake policy node and patch loc policy
+    # create fake policy node and patch loc policy only to fit the general api.
     from contrast.agent.policy.patch_location_policy import PatchLocationPolicy
     from contrast.agent.policy.trigger_node import TriggerNode
 
     trigger_node = TriggerNode(adapter_name, "unused", True, method, "ARG_0")
     patch_policy = PatchLocationPolicy(trigger_node)
 
-    rule = Settings().protect_rules.get(SqlInjection.NAME)
+    rule = Settings().protect_rules.get(SqlInjection.RULE_NAME)
+    if not rule or not rule.enabled:
+        return
+
     rule.protect(patch_policy, sql, args, kwargs)
