@@ -2,6 +2,7 @@
 # Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import contrast
+from contrast.agent import scope as scope_
 from contrast.agent.policy.trigger_node import TriggerNode
 from contrast.agent.middlewares.base_middleware import BaseMiddleware
 from contrast.agent.asgi import (
@@ -49,7 +50,6 @@ class ASGIMiddleware(BaseMiddleware):
                     return
 
             await self.call_without_agent_async(scope, receive, send)
-            return
 
     async def call_with_agent(self, context, request, send) -> None:
         self.log_start_request_analysis()
@@ -92,8 +92,8 @@ class ASGIMiddleware(BaseMiddleware):
 
     async def call_without_agent_async(self, scope, receive, send) -> None:
         super().call_without_agent()
-        await self.asgi_app(scope, receive, send)
-        return
+        with scope_.contrast_scope():
+            await self.asgi_app(scope, receive, send)
 
     @cached_property
     def trigger_node(self):
@@ -110,3 +110,7 @@ class ASGIMiddleware(BaseMiddleware):
             TriggerNode(module, class_name, instance_method, method_name, "RETURN"),
             args,
         )
+
+    @cached_property
+    def name(self):
+        return "asgi"
