@@ -30,9 +30,24 @@ static const char *log_level_map[] = {
     "debug",
 };
 
-void set_logger(PyObject *_logger) {
-    Py_XINCREF(_logger);
-    logger = _logger;
+PyObject *initialize_logger(PyObject *self, PyObject *args, PyObject *kwargs) {
+    PyObject *agent_logger = NULL;
+    char *keywords[] = {"logger", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|O", keywords, &agent_logger)) {
+        /* Exception will propagate, no need to record to stderr */
+        return NULL;
+    }
+
+    /* Called in the scenario this function is called more than once.
+        We decref the previously set logger if there was one set */
+    teardown_logger();
+
+    Py_XINCREF(agent_logger);
+
+    logger = agent_logger;
+
+    Py_RETURN_NONE;
 }
 
 void teardown_logger() {
