@@ -24,7 +24,7 @@
         }                                                                         \
     } while (0);
 
-PyObject *initialize(PyObject *self, PyObject *args, PyObject *kwargs);
+PyObject *initialize(PyObject *, PyObject *);
 PyObject *enable_required_hooks(PyObject *self, PyObject *arg);
 PyObject *install(PyObject *self, PyObject *arg);
 PyObject *disable(PyObject *self, PyObject *args);
