@@ -8,6 +8,7 @@ import contrast.utils.configuration_utils as utils
 from contrast.utils.decorators import cached_property
 from contrast.utils.loggers import DEFAULT_PROGNAME
 from contrast.extern import structlog as logging
+from contrast.utils.singleton import Singleton
 
 logger = logging.getLogger("contrast")
 
@@ -62,6 +63,11 @@ class AgentConfig(object):
 
         self.check_for_api_config()
 
+        # This stores the session ID sent from TS. We have to separate this value from the value set in the yaml config
+        # because the value sent from TS always takes precedence. This value is only sent from TS
+        # in response to ApplicationStartup
+        self._ts_session_id = None
+
     def get(self, key, default_value=None):
         return self._config.get(key, default_value)
 
@@ -186,12 +192,38 @@ class AgentConfig(object):
 
         logger.warning(warn_msg)
 
-    def get_session_id(self):
-        return self.get("application.session_id", "")
+    @property
+    def session_id(self):
+        return (
+            self._ts_session_id
+            if self._ts_session_id is not None
+            else self.get("application.session_id", "")
+        )
+
+    @session_id.setter
+    def session_id(self, session_id):
+        self._ts_session_id = session_id
+        logger.debug("Set session_id to %s", session_id, direct_to_teamserver=1)
 
     def get_session_metadata(self):
         return self.get("application.session_metadata", "")
 
+    @property
+    def app_code(self):
+        return self.get("application.code", "")
+
+    @property
+    def app_metadata(self):
+        return self.get("application.metadata", "")
+
+    @property
+    def app_group(self):
+        return self.get("application.group", "")
+
+    @property
+    def app_tags(self):
+        return self.get("application.tags", "")
+
     @cached_property
     def is_service_bypassed(self):
         bypass = self.get("agent.service.bypass", None)
@@ -201,6 +233,10 @@ class AgentConfig(object):
 
         return is_bypassed
 
+    @cached_property
+    def is_agent_lib_enabled(self):
+        return self.get("agent.python.enable_shared_agent_lib", False)
+
     @cached_property
     def is_request_audit_enabled(self):
         return self.is_service_bypassed and self.get("api.request_audit.enable")
@@ -254,6 +290,10 @@ class ConfigBuilder(object):
         :param config: dict config
         :return: None, config dict is updated pass by reference
         """
+
+        if isinstance(self, Assess):
+            sampling_lock_config = SamplingLockConfig()
+
         for option_name, default_val, type_cast in self.default_options:
 
             dot_alt = self._dot_alternative(option_name)
@@ -266,6 +306,9 @@ class ConfigBuilder(object):
                 config[dot_alt] = env_override
                 continue
 
+            if "sampling" in option_name and config.get(dot_alt):
+                sampling_lock_config.set_sampling_config(option_name.split(".")[-1])
+
             if dot_alt in config:
                 # update the config value with a type-cast value
                 val = config[dot_alt]
@@ -309,6 +352,11 @@ class Agent(ConfigBuilder):
                 default_value=True,
                 type_cast=utils.str_to_bool,
             ),
+            ConfigOption(
+                name="python.enable_shared_agent_lib",
+                default_value=False,
+                type_cast=utils.str_to_bool,
+            ),
             ConfigOption(
                 name="python.enable_rep",
                 default_value=True,
@@ -626,3 +674,21 @@ class Root(ConfigBuilder):
         self.default_options = [
             ConfigOption(name="enable", default_value=True, type_cast=utils.str_to_bool)
         ]
+
+
+class SamplingLockConfig(Singleton):
+    def init(self):
+        self.lock_enable = False
+        self.lock_baseline = False
+        self.lock_frequency = False
+        self.lock_window = False
+
+    def set_sampling_config(self, value):
+        if value == "enable":
+            self.lock_enable = True
+        if value == "baseline":
+            self.lock_baseline = True
+        if value == "request_frequency":
+            self.lock_frequency = True
+        if value == "window_ms":
+            self.lock_window = True
