@@ -15,7 +15,7 @@ from contrast.agent.middlewares.route_coverage.django_routes import (
     create_django_routes,
     build_django_route,
 )
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 
 try:
     from django.urls import get_resolver
@@ -23,6 +23,7 @@ except ImportError:
     from django.core.urlresolvers import get_resolver
 
 from contrast.extern import structlog as logging
+from contrast.utils.decorators import cached_property
 
 logger = logging.getLogger("contrast")
 
@@ -47,17 +48,17 @@ class DjangoWSGIMiddleware(WSGIMiddleware):
 
         super().__init__(wsgi_app, self.app_name)
 
+    @fail_quietly(
+        "Unable to get Django application name", return_value="Django Application"
+    )
     def get_app_name(self):
-        try:
-            from django.conf import settings
+        from django.conf import settings
 
-            wsgi_application = settings.WSGI_APPLICATION
+        wsgi_application = settings.WSGI_APPLICATION
 
-            return wsgi_application.split(".")[0]
-        except Exception:
-            return "Django Application"
+        return wsgi_application.split(".")[0]
 
-    @fail_safely("Unable to get Django view func")
+    @fail_quietly("Unable to get Django view func")
     def get_view_func(self, request):
         from django.conf import settings
 
@@ -74,11 +75,11 @@ class DjangoWSGIMiddleware(WSGIMiddleware):
 
         return match.func
 
-    @fail_safely("Unable to build route", return_value="")
+    @fail_quietly("Unable to build route", return_value="")
     def build_route(self, view_func, url):
         return build_django_route(view_func)
 
-    @fail_safely("Unable to get route coverage", return_value={})
+    @fail_quietly("Unable to get route coverage", return_value={})
     def get_route_coverage(self):
         """
         Route Coverage is the Assess feature that looks for routes generally defined
@@ -86,8 +87,8 @@ class DjangoWSGIMiddleware(WSGIMiddleware):
         """
         return create_django_routes()
 
-    @fail_safely("Failed to run config scanning rules")
-    def scan_configs(self):
+    @fail_quietly("Failed to run config scanning rules")
+    def _scan_configs(self):
         """
         Run config scanning rules for assess
 
@@ -108,8 +109,10 @@ class DjangoWSGIMiddleware(WSGIMiddleware):
         for rule in self.config_rules:
             rule.apply(app_settings, app_config_module)
 
+    @fail_quietly("Failed to _run_router for django middleware")
     def _run_router(self, path):
-        try:
-            return get_resolver().resolve(path or "/")
-        except Exception:
-            return None
+        return get_resolver().resolve(path or "/")
+
+    @cached_property
+    def name(self):
+        return "django"
