@@ -36,7 +36,7 @@ from contrast.agent.policy import patch_manager
 from contrast.agent.assess.policy.source_policy import cs__apply_source
 from contrast.extern.wrapt import register_post_import_hook
 from contrast.utils.assess import stream_utils
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 from contrast.utils.patch_utils import build_and_apply_patch
 
 from contrast.extern import structlog as logging
@@ -44,7 +44,7 @@ from contrast.extern import structlog as logging
 logger = logging.getLogger("contrast")
 
 
-@fail_safely("failed to track cgi.FieldStorage object")
+@fail_quietly("failed to track cgi.FieldStorage object")
 def _track_fieldstorage(fs_obj, context, node):
     """
     Explicitly mark all relevant FieldStorage attributes as sources.
@@ -69,7 +69,7 @@ def _track_fieldstorage(fs_obj, context, node):
         fs_obj.file.cs__source_tags = node.tags
 
 
-@fail_safely("failed to track cgi.MiniFieldStorage object")
+@fail_quietly("failed to track cgi.MiniFieldStorage object")
 def _track_minifieldstorage(fs_obj, context, node):
     """
     Same as _track_fieldstorage, but for MiniFieldStorage. This class only stores a
