@@ -10,14 +10,14 @@ import os
 from contrast.extern.wrapt import register_post_import_hook
 
 from contrast.patches.databases import dbapi2
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 
 PSYCOPG2 = "psycopg2"
 VENDOR = "PostgreSQL"
 
 
 class Psycopg2Patcher(dbapi2.Dbapi2Patcher):
-    @fail_safely("failed to get database inventory information")
+    @fail_quietly("failed to get database inventory information")
     def init_dbinfo(self, connection, connect_args, connect_kwargs):
         """
         Record DB inventory for a Postgres connection.
