@@ -15,7 +15,7 @@ from contrast.agent.policy import patch_manager
 from contrast.agent.assess.policy import trigger_policy
 from contrast.agent.policy.loader import Policy
 from contrast.agent.settings import Settings
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 from contrast.utils.library_reader.patched_state import DistributionContext
 from contrast.utils.library_reader.utils import (
     normalize_file_name,
@@ -119,7 +119,7 @@ def build_new_loaded_files(before_import_sys_modules, after_import_sys_modules):
 LEVEL = 0
 
 
-@fail_safely(
+@fail_quietly(
     "Failed to determine sys module keys to perform analysis on",
     return_value=(False, None, None),
 )
@@ -147,7 +147,7 @@ def pre__import__analysis(
     return True, possible_sys_module_keys, before_import_sys_modules
 
 
-@fail_safely("Failed to analyze assess")
+@fail_quietly("Failed to analyze assess")
 def analyze_assess(method_name, result, args, kwargs):
     if scope.in_trigger_scope() or scope.in_contrast_scope():
         return
@@ -160,7 +160,7 @@ def analyze_assess(method_name, result, args, kwargs):
     trigger_policy.apply(trigger_rule, trigger_nodes, result, args, kwargs)
 
 
-@fail_safely("Failed to determine loaded files")
+@fail_quietly("Failed to determine loaded files")
 def post__import__analysis(
     req_context, before_import_sys_modules, possible_sys_module_keys
 ):
