@@ -5,6 +5,7 @@ from contrast.agent.middlewares.route_coverage.pyramid_routes import PyramidRout
 from contrast.wsgi.middleware import WSGIMiddleware
 from contrast.agent.middlewares.app_finder import get_original_app_or_fail
 from contrast.extern import structlog as logging
+from contrast.utils.decorators import fail_quietly, cached_property
 
 from pyramid.registry import Registry
 
@@ -49,8 +50,12 @@ class PyramidWSGIMiddleware(PyramidRoutesMixin, WSGIMiddleware):
         app_name = self._get_app_name()
         super().__init__(wsgi_app, app_name)
 
+    @fail_quietly(
+        "Unable to get Pyramid application name", return_value="Pyramid Application"
+    )
     def _get_app_name(self):
-        try:
-            return self.registry.package_name
-        except Exception:
-            return "Pyramid Application"
+        return self.registry.package_name
+
+    @cached_property
+    def name(self):
+        return "pyramid"
