@@ -6,6 +6,7 @@ import queue
 
 from .teamserver_messages import BaseTsMessage
 from contrast.agent import scope
+from contrast.agent.disable_reaction import DisableReaction
 from contrast.agent.settings import Settings
 from contrast.reporting import RequestAudit
 from contrast.extern import structlog as logging
@@ -13,9 +14,11 @@ from contrast.utils.singleton import Singleton
 
 logger = logging.getLogger("contrast")
 
+REPORTING_CLIENT_THREAD_NAME = "ContrastReportingClient"
+
 
 class ReportingClient(Singleton, threading.Thread):
-    def __init__(self):
+    def init(self):
         self.message_q = queue.Queue(maxsize=128)
         self.settings = Settings()
         self.request_audit = (
@@ -30,7 +33,10 @@ class ReportingClient(Singleton, threading.Thread):
 
         super().__init__()
 
+    def start(self):
         self.daemon = True
+        self.name = REPORTING_CLIENT_THREAD_NAME
+        super().start()
 
     def run(self):
         with scope.contrast_scope():
@@ -45,7 +51,7 @@ class ReportingClient(Singleton, threading.Thread):
                     pass
 
     def send_message(self, msg):
-        msg_name = type(msg).__name__
+        msg_name = msg.class_name
         try:
             logger.debug(
                 "Sending %s message to Teamserver",
@@ -67,7 +73,7 @@ class ReportingClient(Singleton, threading.Thread):
                         "Certificate configuration is not set properly.",
                         direct_to_teamserver=1,
                     )
-                    self.settings.config.put("enable", False)
+                    DisableReaction.run(self.settings)
                     return None
 
                 cert = (
@@ -95,6 +101,7 @@ class ReportingClient(Singleton, threading.Thread):
             if self.request_audit:
                 self.request_audit.audit(msg, response)
 
+            msg.sent()
             return response
 
         except Exception as e:
@@ -104,7 +111,7 @@ class ReportingClient(Singleton, threading.Thread):
                 e,
                 direct_to_teamserver=1,
             )
-            self.settings.config.put("enable", False)
+            DisableReaction.run(self.settings)
 
         return None
 
@@ -114,8 +121,19 @@ class ReportingClient(Singleton, threading.Thread):
 
         logger.debug(
             "Adding msg to reporting queue: %s",
-            type(msg).__name__,
+            msg.class_name,
             direct_to_teamserver=1,
         )
 
         self.message_q.put(msg)
+
+    def retry_message(self, msg):
+        # Never send a message more than twice (original time plus one retry)
+        # To prevent queue from filling up or causing memory issues.
+        if msg.sent_count < 2:
+            logger.debug(
+                "Re-enqueuing %s message",
+                msg.class_name,
+                direct_to_teamserver=1,
+            )
+            self.add_message(msg)
