@@ -7,10 +7,14 @@ import requests.models
 from requests import put, post
 
 import contrast
+from contrast.agent.disable_reaction import DisableReaction
 from contrast.agent.settings import Settings
 from contrast.api import dtm_pb2
+from contrast.api.architecture_component import ArchitectureComponent
+from contrast.api.attack import Attack
 from contrast.utils.timer import now_ms
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_loudly
+from contrast.utils.object_utils import NOTIMPLEMENTED_MSG
 from contrast.utils.service_util import sleep
 from contrast.utils.string_utils import ensure_string
 from contrast.extern import structlog as logging
@@ -44,8 +48,12 @@ EVENT_TYPE_LOOKUP = {
 }
 
 
+SLEEP_TIME_SECS = 900
+
+
 class BaseTsMessage:
     def __init__(self):
+        self._sent_count = 0
         self.settings = Settings()
 
         self.base_url = f"{self.settings.api_url}/api/ng/"
@@ -71,65 +79,97 @@ class BaseTsMessage:
 
         self.body = ""
 
+    @property
+    def class_name(self):
+        return type(self).__name__.lstrip("_")
+
     @property
     def name(self) -> str:
         """
         Used for request audit filename
         """
-        raise NotImplementedError("This method should be implemented by subclass")
+        raise NotImplementedError(NOTIMPLEMENTED_MSG)
 
     @property
     def path(self) -> str:
         """
         URL path for teamserver; used for formatting as "/api/ng/{path}"
         """
-        raise NotImplementedError("This method should be implemented by subclass")
+        raise NotImplementedError(NOTIMPLEMENTED_MSG)
 
     @property
     def request_method(self) -> str:
-        raise NotImplementedError("This method should be implemented by subclass")
+        raise NotImplementedError(NOTIMPLEMENTED_MSG)
 
     @property
-    def expected_response_code(self):
-        return 200
+    def expected_response_codes(self):
+        return [204]
 
     @property
-    def last_update_or_now(self):
-        return (
-            self.settings.last_update_time_ms
-            if self.settings.last_update_time_ms != 0
-            else now_ms()
-        )
+    def sent_count(self):
+        return self._sent_count
 
-    @fail_safely("Failed to process TS response")
+    def sent(self):
+        self._sent_count += 1
+
+    @fail_loudly("Failed to process TS response")
     def process_response(
         self, response: requests.models.Response, reporting_client
     ) -> None:
-        raise NotImplementedError("This method should be implemented by subclass")
+        raise NotImplementedError(NOTIMPLEMENTED_MSG)
 
-    def process_response_code(self, response):
+    def process_response_code(self, response, reporting_client):
         """
         Return True if response code is expected response code
         """
         if not isinstance(response, requests.models.Response):
             return False
 
-        # TODO: PYT-2120 handle 409, 410, and 5xx properly
-        if response.status_code in (401, 408, 409, 410, 412):
+        logger.debug(
+            "Received %s response code from Teamserver",
+            response.status_code,
+            direct_to_teamserver=1,
+        )
+
+        if response.status_code in (409, 410, 412, 502):
+            # 409: app is archived, 502 app is locked in TS
+            # 410: app is not registered. We could send App startup for for not we won't
+            # 412: API key no longer valid. While spec may say to resend msg in 15mins,
+            #  in reality the app server and agent should simply be restarted.
+            DisableReaction.run(self.settings)
+            return False
+
+        if response.status_code in (401, 408):
+            # 401: Access forbidden because credentials failed to authenticate.
+            # 408: TS Could not create settings in time.
+
+            if isinstance(self, (ServerStart, ApplicationStartup)):
+                DisableReaction.run(self.settings)
+                return False
+
             logger.debug(
-                "Received %s response code from Teamserver. Sleeping for 15 minutes",
-                response.status_code,
+                "Sleeping for 15 minutes",
                 direct_to_teamserver=1,
             )
 
-            sleep(900)
+            sleep(SLEEP_TIME_SECS)
 
+            reporting_client.retry_message(self)
             return False
 
-        if not response.status_code == self.expected_response_code:
+        if response.status_code == 429:
+            sleep_time = int(response.headers.get("Retry-After", SLEEP_TIME_SECS))
+
+            logger.debug("Sleeping for %s seconds", sleep_time, direct_to_teamserver=1)
+
+            sleep(sleep_time)
+
+            reporting_client.retry_message(self)
+
+        if response.status_code not in self.expected_response_codes:
             logger.debug(
                 "Unexpected %s response from TS: %s",
-                type(self).__name__,
+                self.class_name,
                 response.status_code,
                 direct_to_teamserver=1,
             )
@@ -138,11 +178,25 @@ class BaseTsMessage:
         return True
 
 
-class ServerActivity(BaseTsMessage):
+class BaseServerMsg(BaseTsMessage):
+    @fail_loudly(f"Failed to process server settings response")
+    def process_response(self, response, reporting_client):
+        settings = Settings()
+        if not self.process_response_code(response, reporting_client):
+            return
+
+        body = response.json()
+
+        settings.apply_ts_server_settings(body)
+
+        settings.process_ts_reactions(body)
+
+
+class ServerActivity(BaseServerMsg):
     def __init__(self):
         super().__init__()
 
-        self.body = {"lastUpdate": self.last_update_or_now}
+        self.body = {"lastUpdate": self.since_last_update}
 
     @property
     def name(self):
@@ -156,16 +210,65 @@ class ServerActivity(BaseTsMessage):
     def request_method(self):
         return put
 
-    @fail_safely("Failed to process ServerActivity response")
+    @property
+    def expected_response_codes(self):
+        return [200, 304]
+
+    @property
+    def since_last_update(self):
+        """
+        Time in ms since server have been updated.
+        If never updated, then it's been 0ms since then.
+        """
+        if self.settings.last_server_update_time_ms == 0:
+            return 0
+        return now_ms() - self.settings.last_server_update_time_ms
+
+    @fail_loudly("Failed to process ServerActivity response")
     def process_response(self, response, reporting_client):
         settings = Settings()
-        if not self.process_response_code(response):
+
+        # TS will not send server settings unless lastUpdate is >= 5 mins (300_000 ms)
+        # The response in those cases is 304.
+        if (
+            not self.process_response_code(response, reporting_client)
+            or response.status_code == 304
+        ):
             return
 
         body = response.json()
 
         settings.apply_ts_server_settings(body)
 
+        settings.process_ts_reactions(body)
+
+
+class ServerStart(BaseServerMsg):
+    def __init__(self):
+        super().__init__()
+
+        self.body = {
+            "environment": self.settings.config.get("server.environment"),
+            "tags": self.settings.config.get("server.tags"),
+            "version": contrast.__version__,
+        }
+
+    @property
+    def name(self):
+        return "agent_startup"
+
+    @property
+    def path(self):
+        return "/servers/"
+
+    @property
+    def expected_response_codes(self):
+        return [200]
+
+    @property
+    def request_method(self):
+        return put
+
 
 class BaseTsAppMessage(BaseTsMessage):
     def __init__(self):
@@ -183,6 +286,16 @@ class BaseTsAppMessage(BaseTsMessage):
             }
         )
 
+    @property
+    def since_last_update(self):
+        """
+        Time in ms since app settings have been updated.
+        If never updated, then it's been 0ms since then.
+        """
+        if self.settings.last_app_update_time_ms == 0:
+            return 0
+        return now_ms() - self.settings.last_app_update_time_ms
+
 
 class Preflight(BaseTsAppMessage):
     def __init__(self, activity):
@@ -215,9 +328,13 @@ class Preflight(BaseTsAppMessage):
     def request_method(self):
         return put
 
-    @fail_safely("Failed to process Preflight response")
+    @property
+    def expected_response_codes(self):
+        return [200]
+
+    @fail_loudly("Failed to process Preflight response")
     def process_response(self, response, reporting_client):
-        if not self.process_response_code(response):
+        if not self.process_response_code(response, reporting_client):
             return
 
         body = response.text
@@ -260,16 +377,11 @@ class _Traces(BaseTsAppMessage):
     def request_method(self) -> str:
         return put
 
-    @property
-    def expected_response_code(self):
-        return 204
-
-    @fail_safely("Failed to process Traces response")
+    @fail_loudly("Failed to process Traces response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
 
     def _build_body(self, finding, request: dtm_pb2.HttpRequest):
-
         path, _, querystring = request.raw.partition("?")
         return {
             "created": (
@@ -379,7 +491,7 @@ class _Traces(BaseTsAppMessage):
                 for r in finding.routes
             ],
             "ruleId": finding.rule_id,
-            "session_id": "0",  # TODO: PYT-2174 use a real session ID here, not "0"
+            "session_id": self.settings.config.session_id,
             "tags": finding.tags,
             "version": finding.version,
         }
@@ -399,7 +511,7 @@ class ObservedRoute(BaseTsAppMessage):
             "sources": self._parse_sources(observed_route.sources),
         }
 
-        session_id = self.settings.config.get_session_id()
+        session_id = self.settings.config.session_id
         if session_id:
             self.body.update({"session_id": session_id})
 
@@ -419,13 +531,9 @@ class ObservedRoute(BaseTsAppMessage):
     def request_method(self):
         return post
 
-    @property
-    def expected_response_code(self):
-        return 204
-
-    @fail_safely("Failed to process ObservedRoute response")
+    @fail_loudly("Failed to process ObservedRoute response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
 
     def _parse_sources(self, sources):
         new_sources = []
@@ -444,12 +552,13 @@ class ApplicationUpdate(BaseTsAppMessage):
 
         self.headers.update({"Content-Type": "application/json"})
 
-        # activity message sends "components" as "architectures"
+        # activity message sends "components" aka "architectures"
         # so we will not send the "components" field at this time.
 
-        # field "timestamp" represents the last time the app settings were changed.
+        # field "timestamp" represents the amount of time that has passed
+        # since the app settings were changed (not an actual timestamp).
         self.body = {
-            "timestamp": self.last_update_or_now,
+            "timestamp": self.since_last_update,
             "libraries": [
                 lib.to_json(self.settings) for lib in libraries if lib.hash_code
             ],
@@ -463,17 +572,13 @@ class ApplicationUpdate(BaseTsAppMessage):
     def path(self):
         return "update/application"
 
-    @property
-    def expected_response_code(self):
-        return 204
-
     @property
     def request_method(self):
         return put
 
-    @fail_safely("Failed to process ApplicationUpdate response")
+    @fail_loudly("Failed to process ApplicationUpdate response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
 
 
 class ApplicationInventory(BaseTsAppMessage):
@@ -487,7 +592,7 @@ class ApplicationInventory(BaseTsAppMessage):
             "routes": [route.to_json() for route in routes.values()],
         }
 
-        session_id = self.settings.config.get_session_id()
+        session_id = self.settings.config.session_id
         if session_id:
             self.body.update({"session_id": session_id})
 
@@ -497,23 +602,25 @@ class ApplicationInventory(BaseTsAppMessage):
 
     @property
     def path(self):
-        return (
-            f"applications/{self.server_name_b64}/{self.server_path_b64}"
-            f"/{self.server_type_b64}/{self.app_language_b64}/{self.app_name_b64}"
-            "/inventory"
+        return "/".join(
+            [
+                "applications",
+                self.server_name_b64,
+                self.server_path_b64,
+                self.server_type_b64,
+                self.app_language_b64,
+                self.app_name_b64,
+                "inventory",
+            ]
         )
 
     @property
     def request_method(self):
         return post
 
-    @property
-    def expected_response_code(self):
-        return 204
-
-    @fail_safely("Failed to process ObservedRoute response")
+    @fail_loudly("Failed to process ApplicationInventory response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
 
 
 class ObservedLibraryUsage(BaseTsAppMessage):
@@ -534,29 +641,31 @@ class ObservedLibraryUsage(BaseTsAppMessage):
 
                 self.body["observations"].append(message)
 
-        session_id = self.settings.config.get_session_id()
-        if session_id:
-            self.body.update({"session_id": session_id})
-
     @property
     def name(self):
         return "applications-library-usage"
 
     @property
     def path(self):
-        return f"applications/{self.server_name_b64}/{self.server_path_b64}/{self.server_type_b64}/{self.app_language_b64}/{self.app_name_b64}/library-usage"
+        return "/".join(
+            [
+                "applications",
+                self.server_name_b64,
+                self.server_path_b64,
+                self.server_type_b64,
+                self.app_language_b64,
+                self.app_name_b64,
+                "library-usage",
+            ]
+        )
 
     @property
     def request_method(self):
         return post
 
-    @property
-    def expected_response_code(self):
-        return 204
-
-    @fail_safely("Failed to process ObservedLibraryUsage response")
+    @fail_loudly("Failed to process ObservedLibraryUsage response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
 
 
 def _b64url_stripped(header_str):
@@ -582,16 +691,136 @@ class HeartBeat(BaseTsAppMessage):
 
     @property
     def path(self):
-        return f"applications/{self.server_name_b64}/{self.server_path_b64}/{self.server_type_b64}/{self.app_language_b64}/{self.app_name_b64}/heartbeat"
+        return "/".join(
+            [
+                "applications",
+                self.server_name_b64,
+                self.server_path_b64,
+                self.server_type_b64,
+                self.app_language_b64,
+                self.app_name_b64,
+                "heartbeat",
+            ]
+        )
 
     @property
     def request_method(self):
         return post
 
-    @property
-    def expected_response_code(self):
-        return 204
-
-    @fail_safely("Failed to process HeartBeat response")
+    @fail_loudly("Failed to process HeartBeat response")
     def process_response(self, response, reporting_client):
-        self.process_response_code(response)
+        self.process_response_code(response, reporting_client)
+
+
+class ApplicationStartup(BaseTsAppMessage):
+    def __init__(self):
+        super().__init__()
+
+        self.body = {
+            "instrumentation": {
+                "protect": {"enable": self.settings.is_protect_enabled()}
+            }
+        }
+
+        if self.settings.config.get_session_metadata():
+            self.body.update(
+                {"session_metadata": self.settings.config.get_session_metadata()}
+            )
+
+        if self.settings.config.app_metadata:
+            self.body.update({"metadata": self.settings.config.app_metadata})
+
+        if self.settings.config.app_code:
+            self.body.update({"code": self.settings.config.app_code})
+
+        if self.settings.config.session_id:
+            self.body.update({"session_id": self.settings.config.session_id})
+
+        if self.settings.config.app_tags:
+            self.body.update({"tags": self.settings.config.app_tags})
+
+        if self.settings.config.app_group:
+            self.body.update({"group": self.settings.config.app_group})
+
+    @property
+    def name(self):
+        return "applications-create"
+
+    @property
+    def path(self):
+        return "applications/create"
+
+    @property
+    def expected_response_codes(self):
+        return [200]
+
+    @property
+    def request_method(self):
+        return put
+
+    @fail_loudly("Failed to process ApplicationStartup response")
+    def process_response(self, response, reporting_client):
+        if not self.process_response_code(response, reporting_client):
+            return
+
+        body = response.json()
+
+        self.settings.apply_ts_app_settings(body)
+        self.settings.process_ts_reactions(body)
+
+
+class Activity(BaseTsAppMessage):
+    def __init__(self, context):
+        super().__init__()
+
+        self.body = {"lastUpdate": self.since_last_update}
+
+        self.body["inventory"] = {
+            # Used by TeamServer to aggregate counts across a given time period, for Protect and attacker activity.
+            "activityDuration": 0,
+            "components": [],
+        }
+        if context.request.user_agent:
+            self.body["inventory"]["browsers"] = [context.request.user_agent]
+        if context.database_info:
+            for db_info in context.database_info:
+                arch_comp = ArchitectureComponent(db_info).to_json()
+                self.body["inventory"]["components"].append(arch_comp)
+
+        if context.attacks:
+            self.body["defend"] = {"attackers": []}
+
+            for attack_dtm in context.attacks:
+                attack = Attack(attack_dtm, context.activity.http_request).to_json()
+                self.body["defend"]["attackers"].append(
+                    {
+                        "protectionRules": {attack_dtm.rule_id: attack},
+                        "source": {
+                            "ip": context.request.client_addr or "",
+                            "xForwardedFor": context.request.headers.get(
+                                "X-Forwarded-For"
+                            )
+                            or "",
+                        },
+                    }
+                )
+
+    @property
+    def name(self):
+        return "activity-application"
+
+    @property
+    def path(self):
+        return "activity/application"
+
+    @property
+    def request_method(self):
+        return put
+
+    @property
+    def expected_response_codes(self):
+        return [200, 204]
+
+    @fail_loudly("Failed to process Activity response")
+    def process_response(self, response, reporting_client):
+        self.process_response_code(response, reporting_client)
