@@ -3,9 +3,9 @@
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
 import os
 
+from contrast.utils import get_installed_distributions
 from contrast.utils.singleton import Singleton
 from contrast.utils.library_reader.utils import (
-    get_installed_distributions,
     get_top_level_directories,
     get_data,
     get_hash,
