@@ -5,7 +5,7 @@ from collections import OrderedDict
 import inspect
 import sys
 
-from contrast.utils.decorators import fail_safely
+from contrast.utils.decorators import fail_quietly
 from contrast.extern import wrapt
 
 from contrast.agent.policy import patch_manager
@@ -157,7 +157,7 @@ def is_patchable(obj):
     return inspect.isroutine(obj) and not is_unpatchable_builtin_method
 
 
-@fail_safely("Unable to repatch single module", log_level="exception")
+@fail_quietly("Unable to repatch single module")
 def repatch_module(module):
     """Repatch a single module. See docstring for repatch_imported_modules"""
 
@@ -183,7 +183,7 @@ def repatch_module(module):
         patch_manager.patch(module, attr_name)
 
 
-@fail_safely("Unable to patch previously imported modules", log_level="exception")
+@fail_quietly("Unable to patch previously imported modules")
 def repatch_imported_modules():
     """
     Applies patches to modules that were already imported prior to agent startup
