@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright © 2021 Contrast Security, Inc.
+# Copyright © 2022 Contrast Security, Inc.
 # See https://www.contrastsecurity.com/enduser-terms-0317a for more details.
-from contrast.extern import six
-
 from contrast.extern import structlog as logging
 
 logger = logging.getLogger("contrast")
@@ -46,11 +44,11 @@ def truncated_signature(value):
             value = value[:60]
             append_truncate = " [TRUNCATED]"
 
-        value = six.ensure_str(value, errors="ignore")
-        return "{!r}{} (id={})".format(value, append_truncate, id(value))
+        value = ensure_string(value, errors="ignore")
+        return f"{value!r}{append_truncate} (id={id(value)})"
     except Exception as e:
         logger.debug("Failed to truncate string: %s", e)
-        return "(id={})".format(id(value))
+        return f"(id={id(value)})"
 
 
 def index_of_any(value, search_chars):
@@ -86,5 +84,12 @@ def protobuf_safe(value):
     return "" if value is None else value
 
 
-def ensure_string(value):
-    return value if isinstance(value, six.string_types) else value.decode("utf-8")
+def ensure_string(s, encoding="utf-8", errors="ignore"):
+    if not isinstance(s, (str, bytes)):
+        raise TypeError(f"not expecting type {type(s)}")
+
+    return s if isinstance(s, str) else s.decode(encoding, errors)
+
+
+def ensure_binary(s, encoding="utf-8", errors="ignore"):
+    return s if isinstance(s, bytes) else s.encode(encoding, errors)
